==> gbook-0.1.tar.bz2: gbook/captcha-class.php <==
<?php

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * - small captcha class, based on http://kore-nordmann.de/bildmanipulation.html
 */

class captcha {
	var $wordhash;
	var $width;
	var $height;
	var $img;
	var $hash_prefix;
	var $hash_suffix;

	// Constructor
	function captcha()
	{
		$this->wordhash = null;
		$this->width    = 0;
		$this->height   = 0;
		$this->img      = 0;
		$this->hash_prefix = $wgProxyKey;
		$this->hash_suffix = "xyz";
	}

	/**
	 * @return void
	 * @param integer $caplen
	 * @param integer $heigth
	 * @param integer $width
	 * @desc create a new image and initialize $wordhash
	 */
	function generate_image($caplen = 5, $width=135, $height=60)
	{
		$this->height=$height;
		$this->width=$width;

		// define characters of which the captcha can consist
		$alphabet = array(
			'A','B','C','D','E','F','G','H','I','J','K','L',
			'M','N','O','P','Q','R','S','T','U','V','W','X',
			'Y','Z','1','2','3','4','5','6','7','8','9','0');
		
		// generate the captcha string
		$captcha_str = '';
		for ($i=1; $i<=$caplen; $i++)
		{
			// choose randomly a character from alphabet and append it to string
			$chosen = rand(1, sizeof($alphabet));
			$captcha_str .= $alphabet[$chosen-1];
		}

		$this->img = imagecreatetruecolor($this->width, $this->height);
		$back = imagecolorallocate($this->img, 255, 255, 255);
		imagefilledrectangle($this->img, 0, 0, $width, $height, $back);
		imagecolortransparent($this->img, $back);
		imagealphablending($this->img, 1);

		// generate background of randomly built ellipses
		for ($i=0; $i<=160; $i++)
		{
			$r = round(rand( 181, 255));
			$g = round(rand( 181, 255));
			$b = round(rand( 181, 255));
			$color = imagecolorallocatealpha($this->img, $r, $g, $b, 95);
			imagefilledellipse($this->img,
				round(rand(2, $this->width)),
				round(rand(2, $this->height)),
				round(rand(22, $this->width/16)),
				round(rand(22, $this->height/4)), $color);
		}
		$start_x = round($this->width / $caplen);
		$max_font_size = $start_x*0.8;
		$start_x = round(0.5*$start_x);
		$max_x_ofs = round($max_font_size*0.9);

		// set each letter with random angle, size and color
		for ($i=0; $i<=$caplen; $i++)
		{
			$r = round(rand(12, 155));
			$g = round(rand(12, 155));
			$b = round(rand(12, 155));
			$y_pos = ($this->height*0.8);
			$fontsize = round(rand(28, $max_font_size));
			$color = imagecolorallocate($this->img, $r, $g, $b);
			$presign = round(rand(0, 1));
			$angle = round(rand(0, 25));
			if ($presign==true) $angle = -1*$angle;
			ImageTTFText($this->img, $fontsize, $angle,
				$start_x+$i*$max_x_ofs, $y_pos,
				$color, 'captcha.ttf', substr($captcha_str,$i,1));
		}

		$text = $this->hash_prefix . $captcha_str . $this->hash_suffix;
		$this->wordhash = sha1(strtolower($text));
		return;
	}

	/**
	 * @return void
	 * @param integer $frequency
	 * @param integer $intensity
         * @desc add some black-white noise to the image
         * @desc source: http://kore-nordmann.de/bildmanipulation.html
	 */
	function SaltAndPepper($frequency = 5, $intensity = 100) {
	    // Anpassugn der Frequenz
	    $frequency = pow($frequency / 100, 2) * 100;
	
	    // Iterieren ueber das Bild
	    for ($i = 0; $i < $this->width; $i++) {
	        for ($j = 0; $j < $this->height; $j++) {
	
	            // Rauschen setzen?
	            if ($frequency > mt_rand(0, 100)) {
	
	                // Farbe holen
	                $color = imageColorsForIndex($this->img,
				imageColorAt($this->img, $i, $j));
				
	                // Intensitaet zufaellig bestimmen
	                $add = mt_rand(intval($intensity * -2.55),
				intval($intensity * 2.55));
				
	                // Neue Farbe ermitteln
	                $r = max(min($color['red']  + $add, 255), 0);
	                $g = max(min($color['green']+ $add, 255), 0);
	                $b = max(min($color['blue'] + $add, 255), 0);
	                // Pruefen, ob die Farbe existiert, sonst neu in die Farbtabelle einfuegen
	                $color = imageColorExact($this->img, $r, $g, $b);
	                if ($color == -1) {
	                    $color = imageColorAllocate($this->img, $r, $g, $b);
	                }
	                // Pixel schreiben
	                imageSetPixel($this->img, $i, $j, $color);
	            }
	        }
	    }
	}

	/**
	 * @return void
	 * @desc output image with appropriate headers
	 */
	function Border()
	{
		$border = imagecolorallocate($this->img, 166, 166, 166);
		imagerectangle($this->img, 0, 0, $this->width - 1, $this->height - 1, $border);
		return;
	}

	/**
	 * @return void
	 * @param string $type
	 * @desc output image with appropriate headers
	 */
	function send_image($type="png")
	{
		/* output image with appropriate headers */
		setcookie("hash", $this->wordhash, 0, "/");
		header("X-Captcha: WikiCaptcha 0.1 - (C) Tino Reichardt");
		switch($type) {
			case "jpg":
				header("Content-Type: image/jpeg");
				imagejpeg($this->img);
				break;
			case "gif":
				header("Content-Type: image/gif");
				imagegif($this->img);
				break;
			case "png":
			default:
				header("Content-Type: image/png");
				imagepng($this->img);
				break;
		}
		return;
	}

	/**
	 * @return boolean
	 * @param string $word
	 * @desc check hash of password against hash of searched characters
	 */
	function verify($word)
	{
		$text = $this->hash_prefix . $word . $this->hash_suffix;
		if ($_COOKIE['hash'] == sha1(strtolower($text))) {
			return true; /* okay :) */
		}

		return false; /* failed */
	}
}


?>

==> gbook-0.4.tar.bz2: gbook/captcha-class.php <==
<?php

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @package MediaWiki
 * @subpackage Extensions
 * @author Tino Reichardt <milky dash gbook at mcmilk dot de>
 * @copyright © 2006 Tino Reichardt
 * -> based on http://kore-nordmann.de/bildmanipulation.html
 */

class captcha {
	var $wordhash;
	var $width;
	var $height;
	var $img;
	var $hash_prefix;
	var $hash_suffix;

	// Constructor
	function captcha()
	{
		$this->workdir = "";
		$this->wordhash = null;
		$this->width    = 0;
		$this->height   = 0;
		$this->img      = 0;
		$this->hash_prefix = "123";
		$this->hash_suffix = "567";
	}
	
	function set_workdir($path)
	{
		$this->workdir = "$path";
	}

	/**
	 * @return void
	 * @param integer $caplen
	 * @param integer $heigth
	 * @param integer $width
	 * @desc create a new image and initialize $wordhash
	 */
	function generate_image($caplen = 5, $width=135, $height=60)
	{
		$this->height=$height;
		$this->width=$width;

		// define characters of which the captcha can consist
		$alphabet = array(
			'A','B','C','D','E','F','G','H','I','J','K','L',
			'M','N','O','P','Q','R','S','T','U','V','W','X',
			'Y','Z','1','2','3','4','5','6','7','8','9','0');
		
		// generate the captcha string
		$captcha_str = '';
		for ($i=1; $i<=$caplen; $i++)
		{
			// choose randomly a character from alphabet and append it to string
			$chosen = rand(1, sizeof($alphabet));
			$captcha_str .= $alphabet[$chosen-1];
		}

		$this->img = imagecreatetruecolor($this->width, $this->height);
		$back = imagecolorallocate($this->img, 255, 255, 255);
		imagefilledrectangle($this->img, 0, 0, $width, $height, $back);
		imagecolortransparent($this->img, $back);
		imagealphablending($this->img, 1);

		// generate background of randomly built ellipses
		for ($i=0; $i<=160; $i++)
		{
			$r = round(rand( 181, 255));
			$g = round(rand( 181, 255));
			$b = round(rand( 181, 255));
			$color = imagecolorallocatealpha($this->img, $r, $g, $b, 95);
			imagefilledellipse($this->img,
				round(rand(2, $this->width)),
				round(rand(2, $this->height)),
				round(rand(22, $this->width/16)),
				round(rand(22, $this->height/4)), $color);
		}
		$start_x = round($this->width / $caplen);
		$max_font_size = $start_x*0.8;
		$start_x = round(0.5*$start_x);
		$max_x_ofs = round($max_font_size*0.9);

		// set each letter with random angle, size and color
		for ($i=0; $i<=$caplen; $i++)
		{
			$r = round(rand(12, 155));
			$g = round(rand(12, 155));
			$b = round(rand(12, 155));
			$y_pos = ($this->height*0.8);
			$fontsize = round(rand(28, $max_font_size));
			$color = imagecolorallocate($this->img, $r, $g, $b);
			$presign = round(rand(0, 1));
			$angle = round(rand(0, 25));
			if ($presign==true) $angle = -1*$angle;
			if ($this->workdir != "") {
			  $ttf = $this->workdir . '/captcha.ttf';
			} else {
			  $ttf = 'captcha.ttf';
			}
			ImageTTFText($this->img, $fontsize, $angle,
				$start_x+$i*$max_x_ofs, $y_pos,
				$color, $ttf, substr($captcha_str,$i,1));
		}

		$text = $this->hash_prefix . $captcha_str . $this->hash_suffix;
		$this->wordhash = sha1(strtolower($text));
		return;
	}

	/**
	 * @return void
	 * @param integer $frequency
	 * @param integer $intensity
         * @desc add some black-white noise to the image
         * @desc source: http://kore-nordmann.de/bildmanipulation.html
	 */
	function SaltAndPepper($frequency = 5, $intensity = 100) {
	    // Anpassugn der Frequenz
	    $frequency = pow($frequency / 100, 2) * 100;
	
	    // Iterieren ueber das Bild
	    for ($i = 0; $i < $this->width; $i++) {
	        for ($j = 0; $j < $this->height; $j++) {
	
	            // Rauschen setzen?
	            if ($frequency > mt_rand(0, 100)) {
	
	                // Farbe holen
	                $color = imageColorsForIndex($this->img,
				imageColorAt($this->img, $i, $j));
				
	                // Intensitaet zufaellig bestimmen
	                $add = mt_rand(intval($intensity * -2.55),
				intval($intensity * 2.55));
				
	                // Neue Farbe ermitteln
	                $r = max(min($color['red']  + $add, 255), 0);
	                $g = max(min($color['green']+ $add, 255), 0);
	                $b = max(min($color['blue'] + $add, 255), 0);
	                // Pruefen, ob die Farbe existiert, sonst neu in die Farbtabelle einfuegen
	                $color = imageColorExact($this->img, $r, $g, $b);
	                if ($color == -1) {
	                    $color = imageColorAllocate($this->img, $r, $g, $b);
	                }
	                // Pixel schreiben
	                imageSetPixel($this->img, $i, $j, $color);
	            }
	        }
	    }
	}

	/**
	 * @return void
	 * @desc output image with appropriate headers
	 */
	function Border()
	{
		$border = imagecolorallocate($this->img, 166, 166, 166);
		imagerectangle($this->img, 0, 0, $this->width - 1, $this->height - 1, $border);
		return;
	}

	/**
	 * @return void
	 * @param string $type
	 * @desc output image with appropriate headers
	 */
	function send_image($type="png")
	{
		/* output image with appropriate headers */
		setcookie("hash", $this->wordhash, 0, "/");
		header("X-Captcha: WikiCaptcha 0.1 - (C) Tino Reichardt");
		switch($type) {
			case "jpg":
				header("Content-Type: image/jpeg");
				imagejpeg($this->img);
				break;
			case "gif":
				header("Content-Type: image/gif");
				imagegif($this->img);
				break;
			case "png":
			default:
				header("Content-Type: image/png");
				imagepng($this->img);
				break;
		}
		return;
	}

	/**
	 * @return boolean
	 * @param string $word
	 * @desc check hash of password against hash of searched characters
	 */
	function verify($word)
	{
		$text = $this->hash_prefix . $word . $this->hash_suffix;
		if ($_COOKIE['hash'] == sha1(strtolower($text))) {
			return true; /* okay :) */
		}

		return false; /* failed */
	}
}

?>
